#ifndef COIN_SOEVENTMANAGER_H
#define COIN_SOEVENTMANAGER_H

/**************************************************************************\
 * Copyright (c) Kongsberg Oil & Gas Technologies AS
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * Neither the name of the copyright holder nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
\**************************************************************************/

#include <Inventor/SbBasic.h>
#include <Inventor/tools/SbPimplPtr.h>

class SoCamera;
class SbVec2s;
class SoEvent;
class SoNode;
class SbViewportRegion;
class SoHandleEventAction;
class SoScXMLStateMachine;

class COIN_DLL_API SoEventManager {
public:
  SoEventManager(void);
  virtual ~SoEventManager();

  enum NavigationState {
    NO_NAVIGATION,
    JUST_NAVIGATION,
    MIXED_NAVIGATION
  };

  virtual void setSceneGraph(SoNode * const sceneroot);
  virtual SoNode * getSceneGraph(void) const;

  virtual void setCamera(SoCamera * camera);
  virtual SoCamera * getCamera(void) const;

  virtual SbBool processEvent(const SoEvent * const event);

  virtual void setNavigationState(NavigationState state);
  virtual NavigationState getNavigationState(void) const;

  int getNumSoScXMLStateMachines(void) const;
  SoScXMLStateMachine * getSoScXMLStateMachine(int idx) const;
  void addSoScXMLStateMachine(SoScXMLStateMachine * sm);
  void removeSoScXMLStateMachine(SoScXMLStateMachine * sm);

  virtual void setSize(const SbVec2s & newsize);
  virtual void setOrigin(const SbVec2s & newOrigin);
  virtual void setViewportRegion(const SbViewportRegion & newregion);
  const SbViewportRegion & getViewportRegion(void) const;

  virtual void setHandleEventAction(SoHandleEventAction * hea);
  virtual SoHandleEventAction * getHandleEventAction(void) const;

protected:
  virtual SbBool actuallyProcessEvent(const SoEvent * const event);

private:
  class PImpl;
  SbPimplPtr<PImpl> pimpl;

  SoEventManager(const SoEventManager & rhs); // N/A
  SoEventManager & operator = (const SoEventManager & rhs); // N/A

}; // SoEventManager

#endif // !COIN_SOEVENTMANAGER_H
